/**
 * name: @tresjs/post-processing
 * version: v0.6.0
 * (c) 2023
 * description: Post-processing library for TresJS
 * author: Alvaro Saburido <hola@alvarosaburido.dev> (https://github.com/alvarosabu/)
 */
import { inject as j, shallowRef as _, watchEffect as v, onUnmounted as A, watch as w, defineComponent as f, computed as M, provide as X, renderSlot as Y } from "vue";
import { EffectPass as H, BloomEffect as y, GlitchEffect as S, GlitchMode as I, OutlineEffect as P, PixelationEffect as F, DepthOfFieldEffect as R, BlendFunction as U, NoiseEffect as O, EffectComposer as z, RenderPass as K, NormalPass as q, DepthDownsamplingPass as J } from "postprocessing";
import { useTresContext as b, normalizeColor as C, useRenderLoop as Q } from "@tresjs/core";
import { HalfFloatType as D } from "three";
const N = Symbol(), p = (a) => {
  const t = j(N), e = _(null), n = _(null), { scene: o, camera: l } = b();
  v(() => {
    !l.value || !(n != null && n.value) || (n.value.mainCamera = l.value);
  });
  let s = () => {
  };
  return s = v(() => {
    !l.value || !(t != null && t.value) || !o.value || (s(), !n.value && (n.value = a(), e.value = new H(l.value, n.value), t.value.addPass(e.value)));
  }), A(() => {
    var r, i, d;
    e.value && ((r = t == null ? void 0 : t.value) == null || r.removePass(e.value)), (i = n.value) == null || i.dispose(), (d = e.value) == null || d.dispose();
  }), {
    pass: e,
    effect: n
  };
}, G = /([^[.\]])+/g, V = (a, t) => {
  if (!t)
    return;
  const e = Array.isArray(t) ? t : t.match(G);
  return e == null ? void 0 : e.reduce((n, o) => n && n[o], a);
}, T = (a, t, e) => {
  const n = Array.isArray(t) ? t : t.match(G);
  n && n.reduce((o, l, s) => (o[l] === void 0 && (o[l] = {}), s === n.length - 1 && (o[l] = e), o[l]), a);
}, L = (a, t) => {
  const e = { ...a };
  return t.forEach((n) => delete e[n]), e;
}, $ = (a, t, e, n, o = {}) => w(
  a,
  (l) => {
    var s;
    if (t.value)
      if (l === void 0) {
        const r = n();
        T(t.value, e, V(r, e)), (s = r.dispose) == null || s.call(r);
      } else
        T(t.value, e, a());
  },
  o
), E = (a, t, e) => a.map(([n, o]) => $(
  n,
  t,
  o,
  e
)), B = (a, t, e) => Object.keys(a).map((n) => $(
  () => a[n],
  t,
  n,
  e
)), oe = /* @__PURE__ */ f({
  __name: "Bloom",
  props: {
    blendFunction: {},
    intensity: {},
    kernelSize: {},
    luminanceThreshold: {},
    luminanceSmoothing: {},
    mipmapBlur: { type: Boolean, default: void 0 }
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = p(() => new y(e));
    return t({ pass: n, effect: o }), E(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.intensity, "intensity"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.luminanceSmoothing, "luminanceMaterial.smoothing"],
        [() => e.luminanceThreshold, "luminanceMaterial.threshold"]
      ],
      o,
      () => new y()
    ), (l, s) => null;
  }
}), le = /* @__PURE__ */ f({
  __name: "Glitch",
  props: {
    blendFunction: {},
    delay: {},
    duration: {},
    strength: {},
    mode: {},
    active: { type: Boolean },
    ratio: {},
    columns: {},
    chromaticAberrationOffset: {},
    perturbationMap: {},
    dtSize: {}
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = p(() => new S(e));
    return t({ pass: n, effect: o }), v(() => {
      const l = () => {
        if (e.mode !== void 0)
          return e.active === !1 ? I.DISABLED : e.mode;
        const s = new S(), r = s.mode;
        return s.dispose(), r;
      };
      o.value && (o.value.mode = l());
    }), B(
      L(e, ["active", "mode", "blendFunction"]),
      o,
      () => new S()
    ), (l, s) => null;
  }
}), ae = /* @__PURE__ */ f({
  __name: "Outline",
  props: {
    outlinedObjects: {},
    blur: { type: Boolean, default: void 0 },
    xRay: { type: Boolean, default: void 0 },
    kernelSize: {},
    pulseSpeed: {},
    resolutionX: {},
    resolutionY: {},
    edgeStrength: {},
    patternScale: {},
    multisampling: {},
    blendFunction: {},
    patternTexture: {},
    resolutionScale: {},
    hiddenEdgeColor: {},
    visibleEdgeColor: {}
  },
  setup(a, { expose: t }) {
    const e = a, n = (c) => c !== void 0 ? C(c).getHex() : void 0, { camera: o, scene: l } = b(), s = {
      blur: e.blur,
      xRay: e.xRay,
      kernelSize: e.kernelSize,
      pulseSpeed: e.pulseSpeed,
      resolutionX: e.resolutionX,
      resolutionY: e.resolutionY,
      patternScale: e.patternScale,
      edgeStrength: e.edgeStrength,
      blendFunction: e.blendFunction,
      multisampling: e.multisampling,
      patternTexture: e.patternTexture,
      resolutionScale: e.resolutionScale,
      hiddenEdgeColor: n(e.hiddenEdgeColor),
      visibleEdgeColor: n(e.visibleEdgeColor)
    }, { pass: r, effect: i } = p(() => new P(l.value, o.value, s));
    t({ pass: r, effect: i }), w(
      [() => e.outlinedObjects, i],
      // watchEffect is intentionally not used here as it would result in an endless loop
      () => {
        var c;
        (c = i.value) == null || c.selection.set(e.outlinedObjects || []);
      },
      {
        immediate: !0
      }
    );
    const d = M(() => ({
      hiddenEdgeColor: e.hiddenEdgeColor ? C(e.hiddenEdgeColor) : void 0,
      visibleEdgeColor: e.visibleEdgeColor ? C(e.visibleEdgeColor) : void 0
    }));
    return E(
      [
        /* some properties are not updated because of different reasons:
            resolutionX - has no setter in OutlineEffect
            resolutionY - has no setter in OutlineEffect
            blendFunction - has no setter in OutlineEffect
            patternTexture - different type in constructor and in setter
            resolutionScale - has no setter in OutlineEffect
          */
        [() => e.blur, "blur"],
        [() => e.xRay, "xRay"],
        [() => e.pulseSpeed, "pulseSpeed"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.edgeStrength, "edgeStrength"],
        [() => e.patternScale, "patternScale"],
        [() => e.multisampling, "multisampling"],
        [() => d.value.hiddenEdgeColor, "hiddenEdgeColor"],
        [() => d.value.visibleEdgeColor, "visibleEdgeColor"]
      ],
      i,
      () => new P()
    ), (c, x) => null;
  }
}), se = /* @__PURE__ */ f({
  __name: "Pixelation",
  props: {
    granularity: {}
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = p(() => new F(e.granularity));
    return t({ pass: n, effect: o }), B(
      e,
      o,
      () => new F()
    ), (l, s) => null;
  }
}), re = /* @__PURE__ */ f({
  __name: "DepthOfField",
  props: {
    blendFunction: {},
    worldFocusDistance: {},
    worldFocusRange: {},
    focusDistance: {},
    focusRange: {},
    bokehScale: {},
    resolutionScale: {},
    resolutionX: {},
    resolutionY: {}
  },
  setup(a, { expose: t }) {
    const e = a, { camera: n } = b(), { pass: o, effect: l } = p(() => new R(n.value, e));
    return t({ pass: o, effect: l }), E(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.worldFocusDistance, "circleOfConfusionMaterial.worldFocusDistance"],
        [() => e.focusDistance, "circleOfConfusionMaterial.focusDistance"],
        [() => e.worldFocusRange, "circleOfConfusionMaterial.worldFocusRange"],
        [() => e.focusRange, "circleOfConfusionMaterial.focusRange"],
        [() => e.bokehScale, "bokehScale"],
        [() => e.resolutionScale, "blurPass.resolution.scale"],
        [() => e.resolutionX, "resolution.width"],
        [() => e.resolutionY, "resolution.height"]
      ],
      l,
      () => new R()
    ), (s, r) => null;
  }
}), ie = /* @__PURE__ */ f({
  __name: "Noise",
  props: {
    premultiply: { type: Boolean, default: !1 },
    blendFunction: { default: U.SCREEN }
  },
  setup(a, { expose: t }) {
    const e = a, { pass: n, effect: o } = p(() => new O(e));
    return t({ pass: n, effect: o }), B(
      L(e, ["blendFunction"]),
      o,
      () => new O()
    ), (l, s) => null;
  }
});
let m;
function k() {
  var a;
  if (m !== void 0)
    return m;
  try {
    let t;
    const e = document.createElement("canvas");
    return m = !!(window.WebGL2RenderingContext && (t = e.getContext("webgl2"))), t && ((a = t.getExtension("WEBGL_lose_context")) == null || a.loseContext()), m;
  } catch {
    return m = !1;
  }
}
const ue = /* @__PURE__ */ f({
  __name: "EffectComposer",
  props: {
    enabled: { type: Boolean, default: !0 },
    children: {},
    depthBuffer: { type: Boolean, default: void 0 },
    disableNormalPass: { type: Boolean, default: !1 },
    stencilBuffer: { type: Boolean, default: void 0 },
    resolutionScale: {},
    autoClear: { type: Boolean, default: !0 },
    multisampling: { default: 0 },
    frameBufferType: { default: D }
  },
  setup(a) {
    const t = a, { scene: e, camera: n, renderer: o, sizes: l } = b(), s = _(null);
    let r = null, i = null;
    X(N, s);
    const d = () => {
      s.value && (i = new q(e.value, n.value), i.enabled = !1, s.value.addPass(i), t.resolutionScale !== void 0 && k() && (r = new J({
        normalBuffer: i.texture,
        resolutionScale: t.resolutionScale
      }), r.enabled = !1, s.value.addPass(r)));
    };
    v(() => {
      s.value && l.width.value && l.height.value && s.value.setSize(l.width.value, l.height.value);
    });
    const c = M(() => {
      const u = new z(), g = {
        depthBuffer: t.depthBuffer !== void 0 ? t.depthBuffer : u.inputBuffer.depthBuffer,
        stencilBuffer: t.stencilBuffer !== void 0 ? t.stencilBuffer : u.inputBuffer.stencilBuffer,
        multisampling: k() ? 0 : t.multisampling !== void 0 ? t.multisampling : u.multisampling,
        frameBufferType: t.frameBufferType !== void 0 ? t.frameBufferType : D
      };
      return u.dispose(), g;
    }), x = () => {
      !o.value && !e.value && !n.value || (s.value = new z(o.value, c.value), s.value.addPass(new K(e.value, n.value)), t.disableNormalPass || d());
    };
    let h = () => {
    };
    h = w([l.height, l.width], () => {
      !l.height.value && !l.width.value || (v(x), h == null || h());
    }, {
      immediate: !0
    });
    const { onLoop: W } = Q();
    return W(({ delta: u }) => {
      if (t.enabled && o.value && s.value && l.width.value && l.height.value) {
        const g = o.value.autoClear;
        o.value.autoClear = t.autoClear, t.stencilBuffer && !t.autoClear && o.value.clearStencil(), s.value.render(u), o.value.autoClear = g;
      }
    }), A(() => {
      var u;
      (u = s.value) == null || u.dispose();
    }), (u, g) => Y(u.$slots, "default");
  }
});
export {
  oe as Bloom,
  re as DepthOfField,
  ue as EffectComposer,
  le as Glitch,
  ie as Noise,
  ae as Outline,
  se as Pixelation
};
